/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.DecimalFormat;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;

/**
 * Application options.
 *
 * @author Adam Domaski
 * @version 0.1 5/03/2004
 */
public class JetOptions extends JetDialog
{
	/**
	 * Creates new dialog.
	 *
	 * @param owner Parent window.
	 * @param conf Configuration of <b>jET</b> application.
	 */
	public JetOptions(Frame owner, Configuration conf)
	{
		super(owner);

		resource = JetResources.getInstance();
		setTitle( resource.translate("Preferences.title") );

		setResizable(false);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);

		config = conf;

		initialize();
		fillIn();

		setSize(420, 310);
		centerDialog();
		setVisible(true);		
	}

	private void initialize()
	{
		JPanel jPane = new JPanel();
		jPane.setName("PreferencesPanel");
		jPane.setLayout(new BorderLayout());

		JPanel jContainer = createContents();
		jPane.add(jContainer);

		JPanel jControlPanel = new JPanel();
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		jControlPanel.setLayout(gb);
		jControlPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		// Actions
		Action okAction = new OKAction();
		Action caAction = new CancelAction();

		ActionMap actionMap = getLayeredPane().getActionMap();
		actionMap.put("close-it", caAction);
		actionMap.put("enter-it", okAction);

		// Buttons
		JButton okButton = new JButton( resource.translate("Preferences.ok") );
		JButton caButton = new JButton( resource.translate("Preferences.cancel") );

		caButton.addActionListener(caAction);

		okButton.addActionListener(okAction);
		getRootPane().setDefaultButton(okButton);

		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1.0;
		c.anchor = GridBagConstraints.EAST;
		c.insets = new Insets(0, 0, 0, 5);
		gb.setConstraints(okButton, c);
		jControlPanel.add(okButton);

		c.gridx = 1;
		c.gridy = 0;
		c.weightx = 0.0;
		c.insets = new Insets(0,0,0,0);
		gb.setConstraints(caButton, c);
		jControlPanel.add(caButton);

		normalize(okButton, caButton);
		okButton.requestFocus();

		jPane.add(jControlPanel, BorderLayout.SOUTH);
		
		getContentPane().add(jPane);
	}

	private JPanel createContents()
	{
		JTree jTree;
		JPanel jPropertyPanel;

		JPanel jPane = new JPanel();
		jPane.setBorder( BorderFactory.createEmptyBorder(5, 5, 5, 5) );
		jPane.setLayout(new BorderLayout());

		jTree = createPropertiesTree();
		jTree.setName("PropertiesTree");
		controls.putComponent(jTree);
		JScrollPane jScrollPane = new JScrollPane(jTree);
		int height = jScrollPane.getPreferredSize().height;
		jScrollPane.setPreferredSize(new Dimension(120, height));
		jPane.add(jScrollPane, BorderLayout.WEST);

		jPropertyRootPanel = new JPanel();
		jPropertyRootPanel.setLayout(new CardLayout());

		// Tracks
		jPropertyPanel = (JPanel) controls.getComponent("TracksProperty");		
		jPropertyRootPanel.add(jPropertyPanel, "Tracks");

		// Simulator
		jPropertyPanel = (JPanel) controls.getComponent("SimulatorProperty");		
		jPropertyRootPanel.add(jPropertyPanel, "Simulator");

		jPane.add(jPropertyRootPanel);

		return jPane;
	}

	private JTree createPropertiesTree()
	{
		JTree jTree;
		JPanel jPane;
		DefaultMutableTreeNode leaf;
		PropertyPanel propertyPanel;

		DefaultMutableTreeNode top =
			new DefaultMutableTreeNode();

		// Tracks
		leaf = new DefaultMutableTreeNode();
		jPane = createTrackProperty();
		controls.putComponent(jPane);
		propertyPanel = new PropertyPanel("Tracks", jPane);
		leaf.setUserObject(propertyPanel);
		top.add(leaf);

		TreePath treePath = new TreePath(top);
		treePath = treePath.pathByAddingChild(leaf);

		// Simulator
		leaf = new DefaultMutableTreeNode();
		jPane = createSimulatorProperty();
		controls.putComponent(jPane);
		propertyPanel = new PropertyPanel("Simulator", jPane);
		leaf.setUserObject(propertyPanel);
		top.add(leaf);


		jTree = new JTree(top);
		jTree.setRootVisible(false);
		jTree.getSelectionModel().
			setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		DefaultTreeCellRenderer dtcr = 
			new DefaultTreeCellRenderer();
		dtcr.setLeafIcon(null);
		jTree.setCellRenderer(dtcr);
		jTree.setSelectionPath(treePath);
		jTree.addTreeSelectionListener(new TreeHandler());

		return jTree;
	}

	private JPanel createTrackProperty()
	{
		JSlider jSlider;
		JCheckBox jCheckBox;

		JPanel jPane = new JPanel();
		jPane.setName("TracksProperty");
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints gbConstraints = new GridBagConstraints();
		gbConstraints.anchor = GridBagConstraints.WEST;
		gbConstraints.insets = new Insets(0, 1, 0, 1);
		jPane.setLayout(gb);

		jCheckBox = createTrackWiden();
		jCheckBox.setName("TracksPropertyTrackWidenCheckBox");
		controls.putComponent(jCheckBox);
		gbConstraints.gridx = 0;
		gbConstraints.gridy = 0;
		gb.setConstraints(jCheckBox, gbConstraints);
		jPane.add(jCheckBox);

		jSlider = createAdmissableUnbalancedAcceleration();
		jSlider.setName("TracksPropertyUnbalancedAccelerationSlider");
		controls.putComponent(jSlider);
		JPanel admUnbAccPane = new JPanel();
		admUnbAccPane.setBorder(BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(),
			resource.translate("TracksPreferences.unbalancedAccelerationBorder")));
		admUnbAccPane.add(jSlider);

		gbConstraints.gridx = 0;
		gbConstraints.gridy = 1;
		gbConstraints.fill = GridBagConstraints.BOTH;
		gb.setConstraints(admUnbAccPane, gbConstraints);
		jPane.add(admUnbAccPane);

		return jPane;
	}

	private JPanel createSimulatorProperty()
	{
		JLabel jLabel;
		JTextField jText;

		JPanel jPane = new JPanel();
		jPane.setName("SimulatorProperty");
		jPane.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEmptyBorder(3, 3, 3, 3),
			BorderFactory.createTitledBorder(
				BorderFactory.createEtchedBorder(),
				resource.translate("SimulatorPreferences.directories") )) );
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints gbConstraints = new GridBagConstraints();
		gbConstraints.anchor = GridBagConstraints.WEST;
		gbConstraints.insets = new Insets(0, 1, 0, 1);
		jPane.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("TextureDirectoryLabel");
		jLabel.setText( resource.translate("SimulatorPreferences.textures") );
		gbConstraints.gridx = 0;
		gbConstraints.gridy = 0;
		gb.setConstraints(jLabel, gbConstraints);
		jPane.add(jLabel);
		
		jText = new JTextField();
		jText.setName("TextureDirectoryTextBox");
		jText.setColumns(20);
		controls.putComponent(jText);
		gbConstraints.gridx = 0;
		gbConstraints.gridy = 1;
		gb.setConstraints(jText, gbConstraints);
		jPane.add(jText);

		return jPane;
	}

	/**
	 * Creates control for track widen in arcs.
	 */
	private JCheckBox createTrackWiden()
	{
		JCheckBox cb = new JCheckBox();
		cb.setText( resource.translate("TracksPreferences.trackWiden") );

		return cb;
	}

	/**
	 * Creates control for changing admissable unbalanced acceleration.
	 */
	private JSlider createAdmissableUnbalancedAcceleration()
	{
		DecimalFormat dc = new DecimalFormat("0.0");
		Hashtable tables = new Hashtable();
		tables.put( new Integer(0), new JLabel(dc.format(0.4)) );
		tables.put( new Integer(10), new JLabel(dc.format(0.5)) );
		tables.put( new Integer(20), new JLabel(dc.format(0.6)) );
		tables.put( new Integer(30), new JLabel(dc.format(0.7)) );

		JSlider slide = new JSlider(0, 30, 10);
		slide.setMajorTickSpacing(10);
		slide.setMinorTickSpacing(1);
		slide.setLabelTable(tables);
		slide.setPaintTicks(true);
		slide.setPaintLabels(true);
		slide.setSnapToTicks(true);
		return slide;
	}

	private void fillIn()
	{
		JCheckBox jCheckBox;
		JSlider jSlider;
		JTextField jText;

		jCheckBox = (JCheckBox) controls.getComponent("TracksPropertyTrackWidenCheckBox");
		jCheckBox.setSelected( config.getBoolean("TrackWiden") );

		jSlider = (JSlider) controls.getComponent("TracksPropertyUnbalancedAccelerationSlider");
		double d = config.getDouble("AdmissibleUnbalancedAcceleration");
		int value = (int) Math.rint((d - 0.4) * 100.0);
		jSlider.setValue( value );

		jText = (JTextField) controls.getComponent("TextureDirectoryTextBox");
		jText.setText( (String) config.get("TextureDirectory") );
	}

	/*
	 * --------- Property panel ---------
	 */
	
	/**
	 * Class encapsulating property panels.
	 */
	public class PropertyPanel
	{
		public PropertyPanel(String name, JPanel pane)
		{
			this.name = name;
			this.pane = pane;
		}

		public String getName()
		{
			return name;
		}

		public JPanel getPanel()
		{
			return pane;
		}

		public String toString()
		{
			return resource.translate(getName() + "Preferences.name");
		}

		private String name;
		private JPanel pane;
	}

	/*
	 * -------------- Inner classes ------------
	 */

	class TreeHandler implements TreeSelectionListener
	{
		public void valueChanged(TreeSelectionEvent tse)
		{
			TreePath treePath = tse.getPath();
			Object selectedNode = tse.getPath().getLastPathComponent();
			if (selectedNode instanceof DefaultMutableTreeNode)
			{
				PropertyPanel propertyPanel = (PropertyPanel)
					((DefaultMutableTreeNode) selectedNode).getUserObject();

				CardLayout cardLayout = (CardLayout) jPropertyRootPanel.getLayout();
				cardLayout.show(jPropertyRootPanel, propertyPanel.getName());

				jPropertyRootPanel.repaint();
			}
		}
	}

	class OKAction extends AbstractAction
	{
		public void actionPerformed(ActionEvent ae)
		{
			JCheckBox jCheckBox;
			JSlider jSlider;
			JTextField jText;

			jCheckBox = (JCheckBox) controls.getComponent("TracksPropertyTrackWidenCheckBox");
			Boolean b = new Boolean( jCheckBox.isSelected() );
			config.put("TrackWiden", b);

			jSlider = (JSlider) controls.getComponent("TracksPropertyUnbalancedAccelerationSlider");
			double d = (jSlider.getValue() + 40.0) / 100.0;
			config.put("AdmissibleUnbalancedAcceleration", new Double(d));

			jText = (JTextField) controls.getComponent("TextureDirectoryTextBox");
			config.put("TextureDirectory", jText.getText().equals("") ? null : jText.getText());

			dispose();
		}
	}

	class CancelAction extends AbstractAction
	{
		public void actionPerformed(ActionEvent ae)
		{
			dispose();
		}
	}

	/*
	 * ----------- Members --------------
	 */
	
	/** Jet configuration. */
	private Configuration config;

	/** I18n resources. */
	private JetResources resource;

	/** Controls. */
	private ControlMap controls = new ControlMap();

	/** Property panel. */
	private JPanel jPropertyRootPanel;
}
