/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.BasicToggleButtonUI;
import javax.swing.plaf.basic.BasicButtonListener;

/**
 * Toggle button UI for palette buttons.
 *
 * @author Adam Domaski
 * @version 0.1 10/06/2004
 */
public class PaletteButtonUI extends BasicToggleButtonUI
{
	public static ComponentUI createUI(JComponent c)
	{
		return new PaletteButtonUI();
	}

    public void installDefaults(AbstractButton b)
	{
        super.installDefaults(b);
	    selectColor = UIManager.getColor(getPropertyPrefix() + "select");
	}

    protected void installListeners(AbstractButton b)
	{
        ButtonListener listener = createButtonListener((JPaletteButton) b);
        if (listener != null)
		{
            b.putClientProperty(this, listener);
            b.addMouseListener(listener);
            b.addMouseMotionListener(listener);
            b.addFocusListener(listener);
            b.addPropertyChangeListener(listener);
            b.addChangeListener(listener);
        }
    }

    protected void installKeyboardActions(AbstractButton b){
        ButtonListener listener = (ButtonListener) b.getClientProperty(this);
        if (listener != null) {
            listener.installKeyboardActions(b);
        }
    }

	protected ButtonListener createButtonListener(JPaletteButton b)
	{
		return new ButtonListener(b);
	}

    protected void uninstallListeners(AbstractButton b)
	{
        if (listener != null)
		{
            b.removeMouseListener(listener);
            b.removeMouseMotionListener(listener);
            b.removeFocusListener(listener);
            b.removePropertyChangeListener(listener);
            b.removeChangeListener(listener);
        }
        b.putClientProperty(this, null);
		listener = null;
    }

	protected void paintIcon(Graphics g, AbstractButton b, Rectangle iconRect)
	{
		super.paintIcon(g, b, iconRect);
		((JPaletteButton) b).setIconRectangle(iconRect);

		paintHandles(g, (JPaletteButton) b, iconRect.x, iconRect.y);
	}

	protected void paintHandles(Graphics g, JPaletteButton b, int x, int y)
	{
		Image image;
		Point point;

		PaletteButtonModel model = b.getPaletteModel();
		int n = model.getHandlerCount();
		int si = model.getSelectedHandler();

		for (int i = 0; i < n; i++)
		{
			if (i == si)
				image = handlerYellow;
			else
				image = handlerRed;

			point = model.handlerToPoint(i);
			
			g.drawImage(image, point.x + x-3, point.y + y-3, b);
		}
	}

	protected void paintButtonPressed(Graphics g, AbstractButton b)
	{
        if ( b.isContentAreaFilled() )
		{
            Dimension size = b.getSize();
			g.setColor(getSelectColor());
			g.fillRect(0, 0, size.width, size.height);
		}
	}

    protected Color getSelectColor()
	{
		return selectColor;
    }

	/*
	 * ------------- Listeners ----------------
	 */
	
	public class ButtonListener extends BasicButtonListener
	{
		public ButtonListener(JPaletteButton b)
		{
			super(b);
		}

		public void mousePressed(MouseEvent me)
		{
			if (SwingUtilities.isLeftMouseButton(me))
			{
				JPaletteButton b = (JPaletteButton) me.getComponent();
				PaletteButtonModel model = b.getPaletteModel();
				Rectangle rect = b.getIconRectangle();
				
				Point p = me.getPoint();
				p.translate(-rect.x, -rect.y);
				int index = model.pointToHandler(p);
				if (index != -1)
				{
					model.setSelectedHandler(index);
					b.repaint();
				}
				else
					super.mousePressed(me);
			}
		}

		public void mouseReleased(MouseEvent me)
		{
			if (SwingUtilities.isLeftMouseButton(me))
			{
				JPaletteButton b = (JPaletteButton) me.getComponent();
				PaletteButtonModel model = b.getPaletteModel();
				Rectangle rect = b.getIconRectangle();
				
				Point p = me.getPoint();
				p.translate(-rect.x, -rect.y);
				int index = model.pointToHandler(p);
				if (index == -1)
					super.mouseReleased(me);
			}
		}
	}

	/*
	 * --------------- Members ----------------
	 */

	/** Red handler image. */
	private static final Image handlerRed = new ImageIcon("images/se_handler_red.png").getImage();

	/** Base point image. */
	private static final Image handlerYellow = new ImageIcon("images/se_handler_yellow.png").getImage();

	/** Track button listener. */
	private ButtonListener listener = null;

    protected Color selectColor;
}
