/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.sce;

import javax.swing.plaf.ComponentUI;
import java.awt.geom.Point2D;
import javax.vecmath.Point3d;

/**
 * Dummy scenery editor user interface.
 *
 * @author Adam Domaski
 * @version 0.1 22/08/2003
 */
public abstract class SceneryEditorUI extends ComponentUI
{
	/**
	 * Gets an editor kit for scenery editor.
	 * <p>Editor kit has some useful tools.</p>
	 *
	 * @param sc Scenery editor.
	 * @return Editor kit for given scenery editor.
	 * @see EditorKit
	 */
	public abstract EditorKit getEditorKit(JSceneryEditor sc);

	/**
	 * Translates model coordinates to view coordinates for specfied
	 * scenery editor.
	 *
	 * @param editor Scenery editor.
	 * @param p Point in model coordinates.
	 * @return Point in component coordinates.
	 */
	public abstract Point2D modelToView(JSceneryEditor editor, Point3d p);

	/**
	 * Translates view coordinates to model.
	 *
	 * @param editor Editor.
	 * @param p Point in view coordinates.
	 * @return Point in model coordinates.
	 */
	public abstract Point3d viewToModel(JSceneryEditor editor, Point2D p);
}
