/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.tracks;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.vecmath.*;

import org.jet.*;
import org.jet.sce.*;

/**
 * Custom straight track properties dialog.
 *
 * @author Adam Domaski
 * @version 0.1 11/03/2004
 */
public class CustomStraightProperties extends TrackProperties
{
	public CustomStraightProperties(Jet app)
	{
		super(app.getFrame());

		application = app;
		editable = true;

		resource = JetResources.getInstance();
		setTitle( resource.translate("CustomStraightProperties.title") );

		setResizable(false);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);

		initialize();
		fillIn();

		pack();
		centerDialog();
		setVisible(true);
	}

	/**
	 * Initialize dialog controls.
	 */
	private void initialize()
	{
		JPanel pane = new JPanel();
		pane.setLayout(new BorderLayout());

		JPanel container = createContent();
		pane.add(container);

		JPanel controlPanel = new JPanel();		
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		controlPanel.setLayout(gb);
		controlPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		// Actions
		Action okAction = new OKAction();
		Action caAction = new CancelAction();

		ActionMap actionMap = getLayeredPane().getActionMap();
		actionMap.put("close-it", caAction);
		actionMap.put("enter-it", okAction);

		JButton okButton = new JButton( resource.translate("TrackProperties.ok") );
		JButton caButton = new JButton( resource.translate("TrackProperties.cancel") );

		caButton.addActionListener(caAction);

		getRootPane().setDefaultButton(okButton);
		okButton.addActionListener(okAction);

		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1.0;
		c.anchor = GridBagConstraints.EAST;
		c.insets = new Insets(0, 0, 0, 5);
		gb.setConstraints(okButton, c);
		controlPanel.add(okButton);

		c.gridx = 1;
		c.gridy = 0;
		c.weightx = 0.0;
		c.insets = new Insets(0, 0, 0, 0);
		gb.setConstraints(caButton, c);
		controlPanel.add(caButton);

		normalize(okButton, caButton);
		okButton.requestFocus();

		pane.add(controlPanel, BorderLayout.SOUTH);

		getContentPane().add(pane);
	}

	/**
	 * Creates main content.
	 */
	private JPanel createContent()
	{
		JPanel jPanel;
		JLabel jLabel;
		JTextField jText;

		JPanel pane = new JPanel();
		pane.setName("TrackPropertiesPanel");
		pane.setBorder( BorderFactory.createEmptyBorder(1, 1, 1, 1) );
		
		JTabbedPane jTabbedPane = new JTabbedPane();
		jTabbedPane.setName("TrackPropertiesTabbedPane");

		JPanel jGeneralPanel = new JPanel();
		jGeneralPanel.setName("GeneralPanel");
		jGeneralPanel.setBorder( BorderFactory.createEmptyBorder(3, 3, 3, 3) );
		jGeneralPanel.setLayout(new GridLayout(2, 1, 0, 5));

		jPanel = new JPanel();
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		Insets insets = new Insets(1, 2, 1, 2);
		jPanel.setLayout(gb);
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		
		jLabel = new JLabel();
		jLabel.setName("NameLabel");
		jLabel.setText( resource.translate("TrackProperties.name") );
		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.WEST;
		c.insets = insets;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createName();
		jText.setName("NameTextBox");
		jText.setColumns(10);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("LengthLabel");
		jLabel.setText( resource.translate("TrackProperties.length") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createLength();
		jText.setName("LengthTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("RadiusLabel");
		jLabel.setText( resource.translate("TrackProperties.radius") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRadius();
		jText.setName("RadiusTextBox");
		jText.setColumns(7);
		jText.setEnabled(false);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("WidthLabel");
		jLabel.setText( resource.translate("TrackProperties.width") );
		c.gridx = 0;
		c.gridy = 3;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createWidth();
		jText.setName("WidthTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 3;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("FrictionLabel");
		jLabel.setText( resource.translate("TrackProperties.friction") );
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createFriction();
		jText.setName("FrictionTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("TapLabel");
		jLabel.setText( resource.translate("TrackProperties.tap") );
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTap();
		jText.setName("TapTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("QualityLabel");
		jLabel.setText( resource.translate("TrackProperties.quality") );
		c.gridx = 2;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createQuality();
		jText.setName("QualityTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("DamageLabel");
		jLabel.setText( resource.translate("TrackProperties.damage") );
		c.gridx = 2;
		c.gridy = 3;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createDamage();
		jText.setName("DamageTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 3;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jPanel.setFocusTraversalPolicy(new GeneralTabFocusTraversal());
		jGeneralPanel.add(jPanel);

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("EnviromentLabel");
		jLabel.setText( resource.translate("TrackProperties.enviroment") );
		c.gridx = 0;
		c.gridy = 0;
		c.insets = insets;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		JComboBox jComboBox = new JComboBox();
		jComboBox.setName("EnviromentComboBox");
		jComboBox.addItem(Track.BANK);
		jComboBox.addItem(Track.BRIDGE);
		jComboBox.addItem(Track.CANYON);
		jComboBox.addItem(Track.FLAT);
		jComboBox.addItem(Track.MOUNTAINS);
		jComboBox.addItem(Track.TUNNEL);
		controls.putComponent(jComboBox);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jComboBox, c);
		jPanel.add(jComboBox);

		jGeneralPanel.add(jPanel);

		jTabbedPane.addTab(resource.translate("TrackProperties.general"), jGeneralPanel);

		JPanel jTexturesPanel = new JPanel();
		jTexturesPanel.setName("TexturesPanel");
		jTexturesPanel.setBorder( BorderFactory.createEmptyBorder(3, 3, 3, 3) );
		jTexturesPanel.setLayout(new GridLayout(2, 1, 0, 5));

		/* Tekstury - grny panel */

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("VisibilityLabel");
		jLabel.setText( resource.translate("TrackProperties.visibility") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		JPanel jVisibilityPanel = new JPanel();
		jVisibilityPanel.setBorder( BorderFactory.createEtchedBorder() );
		jVisibilityPanel.setLayout( new GridLayout(1, 2) );

		JRadioButton jRadioButton;
		ButtonGroup bGroup = new ButtonGroup();

		jRadioButton = new JRadioButton();
		jRadioButton.setName("VisibleRadioButton");
		jRadioButton.setText( resource.translate("TrackProperties.visible") );
		controls.putComponent(jRadioButton);
		bGroup.add(jRadioButton);
		jVisibilityPanel.add(jRadioButton);

		jRadioButton = new JRadioButton();
		jRadioButton.setName("InvisibleRadioButton");
		jRadioButton.setText( resource.translate("TrackProperties.invisible") );
		controls.putComponent(jRadioButton);
		bGroup.add(jRadioButton);
		jVisibilityPanel.add(jRadioButton);

		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jVisibilityPanel, c);
		jPanel.add(jVisibilityPanel);
		
		jTexturesPanel.add(jPanel);

		/* Tekstury - dolny panel */

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("TextureLabel");
		jLabel.setText( resource.translate("TrackProperties.texture") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jComboBox = createTexture();
		jComboBox.setName("TextureComboBox");
		controls.putComponent(jComboBox);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jComboBox, c);
		jPanel.add(jComboBox);

		jLabel = new JLabel();
		jLabel.setName("TextureLengthLabel");
		jLabel.setText( resource.translate("TrackProperties.textureLength") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureLength();
		jText.setName("TextureLengthEditBox");
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("SecondTextureLabel");
		jLabel.setText( resource.translate("TrackProperties.secondTexture") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jComboBox = createSecondTexture();
		jComboBox.setName("SecondTextureComboBox");
		controls.putComponent( jComboBox );
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jComboBox, c);
		jPanel.add(jComboBox);

		jLabel = new JLabel();
		jLabel.setName("TextureWidthLabel");
		jLabel.setText( resource.translate("TrackProperties.textureWidth") );
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureWidth();
		jText.setName("TextureWidthTextBox");
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("TextureHeightLabel");
		jLabel.setText( resource.translate("TrackProperties.textureHeight") );
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureHeight();
		jText.setName("TextureHeightTextBox");
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("TextureSlopeLabel");
		jLabel.setText( resource.translate("TrackProperties.textureSlope") );
		c.gridx = 2;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureSlope();
		jText.setName("TextureSlopeTextBox");
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 3;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jTexturesPanel.add(jPanel);

		jTabbedPane.addTab(resource.translate("TrackProperties.textures"), jTexturesPanel);

		/* Geometria */
		JPanel jGeometryPanel = new JPanel();
		jGeometryPanel.setName("GeometryPanel");
		jGeometryPanel.setBorder( BorderFactory.createEmptyBorder(3, 3, 3, 3) );
		jGeometryPanel.setLayout(new GridLayout(2, 1, 0, 5));

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("Point1Label");
		jLabel.setText( resource.translate("TrackProperties.point1") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Point1XTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point1YTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point1ZTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Vector1Label");
		jLabel.setText( resource.translate("TrackProperties.vector1") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Vector1XTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector1YTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector1ZTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Roll1Label");
		jLabel.setText( resource.translate("TrackProperties.roll1") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRoll();
		jText.setName("Roll1TextBox");
		jText.setColumns(6);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jGeometryPanel.add(jPanel);

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("Vector2Label");
		jLabel.setText( resource.translate("TrackProperties.vector2") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Vector2XTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector2YTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector2ZTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Point2Label");
		jLabel.setText( resource.translate("TrackProperties.point2") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Point2XTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point2YTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point2ZTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Roll2Label");
		jLabel.setText( resource.translate("TrackProperties.roll2") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRoll();
		jText.setName("Roll2TextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jGeometryPanel.add(jPanel);

		jTabbedPane.addTab(resource.translate("TrackProperties.geometry"), jGeometryPanel);

		pane.add(jTabbedPane);

		return pane;
	}

	private void fillIn()
	{
		JTextField			jText;
		JFormattedTextField jFormat;
		JRadioButton		jRadioButton;
		JComboBox			jComboBox;

		Configuration config = application.config;

		jFormat = (JFormattedTextField) controls.getComponent("LengthTextBox");
		jFormat.setValue( config.get("CustomStraightProperties.length") );

		jFormat = (JFormattedTextField) controls.getComponent("WidthTextBox");
		jFormat.setValue( config.get("CustomStraightProperties.width") );

		jFormat = (JFormattedTextField) controls.getComponent("TapTextBox");
		jFormat.setValue( config.get("CustomStraightProperties.tap") );

		jFormat = (JFormattedTextField) controls.getComponent("FrictionTextBox");
		jFormat.setValue( config.get("CustomStraightProperties.friction") );

		jFormat = (JFormattedTextField) controls.getComponent("QualityTextBox");
		jFormat.setValue( config.get("CustomStraightProperties.quality") );

		jFormat = (JFormattedTextField) controls.getComponent("DamageTextBox");
		jFormat.setValue( config.get("CustomStraightProperties.damage") );

		jComboBox = (JComboBox) controls.getComponent("EnviromentComboBox");
		jComboBox.setSelectedItem( config.get("CustomStraightProperties.enviroment") );

		boolean bool = 
			((Boolean) config.get("CustomStraightProperties.visibility")).booleanValue();
		if (bool)
			jRadioButton = (JRadioButton) controls.getComponent("VisibleRadioButton");
		else
			jRadioButton = (JRadioButton) controls.getComponent("InvisibleRadioButton");
		jRadioButton.setSelected( true );

		jComboBox = (JComboBox) controls.getComponent("TextureComboBox");
		jComboBox.setSelectedItem( config.get("CustomStraightProperties.texture") );

		jFormat = (JFormattedTextField) controls.getComponent("TextureLengthEditBox");
		jFormat.setValue( config.get("CustomStraightProperties.texLength") );

		jComboBox = (JComboBox) controls.getComponent("SecondTextureComboBox");
		jComboBox.setSelectedItem( config.get("CustomStraightProperties.secondTexture") );

		jFormat = (JFormattedTextField) controls.getComponent("TextureWidthTextBox");
		jFormat.setValue( config.get("CustomStraightProperties.texWidth") );

		jFormat = (JFormattedTextField) controls.getComponent("TextureHeightTextBox");
		jFormat.setValue( config.get("CustomStraightProperties.texHeight") );

		jFormat = (JFormattedTextField) controls.getComponent("TextureSlopeTextBox");
		jFormat.setValue( config.get("CustomStraightProperties.texSlope") );

		jFormat = (JFormattedTextField) controls.getComponent("Roll1TextBox");
		jFormat.setValue( config.get("CustomStraightProperties.roll1") );

		jFormat = (JFormattedTextField) controls.getComponent("Roll2TextBox");
		jFormat.setValue( config.get("CustomStraightProperties.roll2") );

		setConstraints();
	}

	/**
	 * Sets the constraints for text fields.
	 */
	private void setConstraints()
	{
		JFormattedTextField jFormatText;
		
		jFormatText = (JFormattedTextField) controls.getComponent("Roll1TextBox");
		setConstraints(jFormatText, new Double(-Track.ROLL), new Double(Track.ROLL));
		jFormatText = (JFormattedTextField) controls.getComponent("Roll2TextBox");
		setConstraints(jFormatText, new Double(-Track.ROLL), new Double(Track.ROLL));
	}

	/*
	 * ---------- Inner class -----------
	 */
	
	class OKAction extends AbstractAction
	{
		public void actionPerformed(ActionEvent ae)
		{
			double					d;
			int						i;
			JFormattedTextField		jFormat;
			JTextField				jText;
			JRadioButton			jButton;
			JComboBox				jComboBox;

			TrackFactory factory = TrackFactory.getInstance();

			jFormat = (JFormattedTextField) controls.getComponent("LengthTextBox");
			double length = ((Number) jFormat.getValue()).doubleValue();

			Track track = factory.createStraight(length);

			// Uzupenij pozostae dane
			jText = (JTextField) controls.getComponent("NameTextBox");
			String name = jText.getText();
			if (name.equals(""))
				name = null;
			track.setName( name );

			track.setRadius(0.0);

			// Width
			jFormat = (JFormattedTextField) controls.getComponent("WidthTextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			track.setWidth( d );

			// Tap
			jFormat = (JFormattedTextField) controls.getComponent("TapTextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			track.setTap( d );

			// Friction
			jFormat = (JFormattedTextField) controls.getComponent("FrictionTextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			track.setFriction( d );

			// Damage
			jFormat = (JFormattedTextField) controls.getComponent("DamageTextBox");
			i = ((Number) jFormat.getValue()).intValue();
			track.setDamage( i );

			// Quality
			jFormat = (JFormattedTextField) controls.getComponent("QualityTextBox");
			i = ((Number) jFormat.getValue()).intValue();
			track.setQuality( i );

			// Enviroment
			jComboBox = (JComboBox) controls.getComponent("EnviromentComboBox");
			track.setEnviroment( (String) jComboBox.getSelectedItem() );

			// Visibility
			jButton = (JRadioButton) controls.getComponent("VisibleRadioButton");
			track.setVisible( jButton.isSelected() );

			// Texture
			jComboBox = (JComboBox) controls.getComponent("TextureComboBox");
			track.setTexture( (String) jComboBox.getSelectedItem() );

			// Second texture
			jComboBox = (JComboBox) controls.getComponent("SecondTextureComboBox");
			track.setSecondTexture( (String) jComboBox.getSelectedItem() );

			// Texture length
			jFormat = (JFormattedTextField) controls.getComponent("TextureLengthEditBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			track.setTextureLength(d);

			// Texture width
			jFormat = (JFormattedTextField) controls.getComponent("TextureWidthTextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			track.setTextureWidth(d);

			// Texture height
			jFormat = (JFormattedTextField) controls.getComponent("TextureHeightTextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			track.setTextureHeight(d);

			// Texture slope
			jFormat = (JFormattedTextField) controls.getComponent("TextureSlopeTextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			track.setTextureSlope(d);

			// Roll 1
			jFormat = (JFormattedTextField) controls.getComponent("Roll1TextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			track.setRoll1(d);

			// Roll 2
			jFormat = (JFormattedTextField) controls.getComponent("Roll2TextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			track.setRoll2(d);

			// Wsad to teraz do scenerii
			SceneryFrame frame = application.getTopmostFrame();
			if (frame != null)
			{
				JSceneryEditor editor = frame.getSceneryEditor();

				BasePoint basePoint = editor.getBasePoint();
				
				if (basePoint != null)
				{
				// Wykonaj obrt toru
				double alpha = - basePoint.getAlpha();
				double beta = - basePoint.getBeta();

				Matrix4d rotMatrix = new Matrix4d();
				rotMatrix.rotY(alpha);

				rotMatrix.transform( track.getPoint1() );
				rotMatrix.transform( track.getVector1() );
				rotMatrix.transform( track.getVector2() );
				rotMatrix.transform( track.getPoint2() );

				AxisAngle4d rotAxis = 
					new AxisAngle4d(track.getPoint2().z, 0.0, -track.getPoint2().x, beta);
				rotMatrix = new Matrix4d();
				rotMatrix.set(rotAxis);

				rotMatrix.transform( track.getPoint1() );
				rotMatrix.transform( track.getVector1() );
				rotMatrix.transform( track.getVector2() );
				rotMatrix.transform( track.getPoint2() );

				// wykonaj przesunicie toru
				Matrix4d traMatrix = new Matrix4d();
				traMatrix.setIdentity();
				traMatrix.setTranslation( new Vector3d(basePoint.getPoint()) );

				traMatrix.transform( track.getPoint1() );
				traMatrix.transform( track.getVector1() );
				traMatrix.transform( track.getVector2() );
				traMatrix.transform( track.getPoint2() );

				Scenery scenery = editor.getScenery();
				Object[] obj = new Object[1];
				obj[0] = track;
				scenery.insert(obj);

				// Zaznacz nowy obiekt
				editor.setSelectedObject(track);

				// Ustaw nowy punkt bazowy
				Point3d point = track.getPoint2();
				Vector3d resolved = null;

				if (!track.isStraight())
				{
					resolved = new Vector3d( track.getVector2() );
					resolved.negate();
				}
				else
				{
					Point3d p1 = track.getPoint1();
					resolved = new Vector3d();
					resolved.x = point.x - p1.x;
					resolved.y = point.y - p1.y;
					resolved.z = point.z - p1.z;
				}

				BasePoint nBasePoint = new BasePoint(point, resolved);
				editor.setBasePoint(nBasePoint);
				}
			}

			saveConfiguration(track);
			dispose();
		}

		private void saveConfiguration(Track track)
		{
			Configuration config = application.config;

			config.put("CustomStraightProperties.length", new Double(track.getLength()));
			config.put("CustomStraightProperties.width", new Double(track.getWidth()));
			config.put("CustomStraightProperties.friction", new Double(track.getFriction()));
			config.put("CustomStraightProperties.tap", new Double(track.getTap()));
			config.put("CustomStraightProperties.damage", new Integer(track.getDamage()));
			config.put("CustomStraightProperties.quality", new Integer(track.getQuality()));
			config.put("CustomStraightProperties.enviroment", track.getEnviroment());
			config.put("CustomStraightProperties.visibility", new Boolean(track.isVisible()));
			config.put("CustomStraightProperties.texture", track.getTexture());
			config.put("CustomStraightProperties.texLength", new Double(track.getTextureLength()));
			config.put("CustomStraightProperties.secondTexture", track.getSecondTexture());
			config.put("CustomStraightProperties.texWidth", new Double(track.getTextureWidth()));
			config.put("CustomStraightProperties.texHeight", new Double(track.getTextureHeight()));
			config.put("CustomStraightProperties.texSlope", new Double(track.getTextureSlope()));
			config.put("CustomStraightProperties.roll1", new Double(track.getRoll1()));
			config.put("CustomStraightProperties.roll2", new Double(track.getRoll2()));
		}
	}

	class CancelAction extends AbstractAction
	{
		public void actionPerformed(ActionEvent ae)
		{
			dispose();
		}
	}

	/*
	 * ------------- Members -------------
	 */
	
	/** Application. */
	private Jet application;
}
