/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.tracks;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.JTextComponent;
import javax.vecmath.*;

import org.jet.*;
import org.jet.sce.*;

/**
 * Generic switch properties dialog.
 *
 * @author Adam Domaski
 * @version 0.1 20/02/2004
 */
public class SwitchProperties extends TrackProperties
{
	/**
	 * Constructs generic switch properties dialog and shows it.
	 *
	 * @param frame Parent frame.
	 * @param editor Topmost scenery editor.
	 */
	public SwitchProperties(Frame frame, JSceneryEditor editor)
	{
		super(frame);

		resource = JetResources.getInstance();
		setTitle( resource.translate("SwitchProperties.title") );

		setResizable(false);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);

		this.editable = editor.isEditable();
		this.editor = editor;

		initialize();
		fillIn((Switch) editor.getSelectedObject());

		pack();
		centerDialog();
		setVisible(true);
	}

	/**
	 * Constructs empty dialog box.
	 *
	 * @param frame Parent frame.
	 */
	protected SwitchProperties(Frame frame)
	{
		super(frame);
	}

	/*
	 * ------------ Methods ------------
	 */

	/**
	 * Initialize dialog controls.
	 */
	private void initialize()
	{
		JPanel pane = new JPanel();
		pane.setLayout(new BorderLayout());

		JPanel container = createContent();
		pane.add(container);

		JPanel controlPanel = new JPanel();		
		Action caAction = new CancelAction();
		ActionMap actionMap = getLayeredPane().getActionMap();
		actionMap.put("close-it", caAction);
		if (isDialogEditable())
		{
			GridBagLayout gb = new GridBagLayout();
			GridBagConstraints c = new GridBagConstraints();
			controlPanel.setLayout(gb);
			controlPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

			// Actions
			Action okAction = new OKAction();

			actionMap.put("enter-it", okAction);

			JButton okButton = new JButton( resource.translate("SwitchProperties.ok") );
			JButton caButton = new JButton( resource.translate("SwitchProperties.cancel") );

			caButton.addActionListener(caAction);

			getRootPane().setDefaultButton(okButton);
			okButton.addActionListener(okAction);

			c.gridx = 0;
			c.gridy = 0;
			c.weightx = 1.0;
			c.anchor = GridBagConstraints.EAST;
			c.insets = new Insets(0, 0, 0, 5);
			gb.setConstraints(okButton, c);
			controlPanel.add(okButton);

			c.gridx = 1;
			c.gridy = 0;
			c.weightx = 0.0;
			c.insets = new Insets(0, 0, 0, 0);
			gb.setConstraints(caButton, c);
			controlPanel.add(caButton);

			normalize(okButton, caButton);
			okButton.requestFocus();
		}
		else
		{
			JButton btn = new JButton( resource.translate("SwitchProperties.close") );
			getRootPane().setDefaultButton(btn);
			btn.addActionListener(caAction);
			controlPanel.add(btn);
		}
		pane.add(controlPanel, BorderLayout.SOUTH);

		getContentPane().add(pane);
	}

	/**
	 * Creates main content.
	 */
	private JPanel createContent()
	{
		JPanel jPanel;
		JLabel jLabel;
		JTextField jText;

		JPanel pane = new JPanel();
		pane.setName("SwitchPropertiesPanel");
		pane.setBorder( BorderFactory.createEmptyBorder(1, 1, 1, 1) );
		
		JTabbedPane jTabbedPane = new JTabbedPane();
		jTabbedPane.setName("SwitchPropertiesTabbedPane");

		JPanel jGeneralPanel = new JPanel();
		jGeneralPanel.setName("GeneralPanel");
		jGeneralPanel.setBorder( BorderFactory.createEmptyBorder(3, 3, 3, 3) );
		jGeneralPanel.setLayout(new GridLayout(2, 1, 0, 5));

		jPanel = new JPanel();
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		Insets insets = new Insets(1, 2, 1, 2);
		jPanel.setLayout(gb);
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		
		jLabel = new JLabel();
		jLabel.setName("NameLabel");
		jLabel.setText( resource.translate("SwitchProperties.name") );
		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.WEST;
		c.insets = insets;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createName();
		jText.setName("NameTextBox");
		jText.setColumns(10);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("LengthLabel");
		jLabel.setText( resource.translate("SwitchProperties.length") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createLength();
		jText.setName("LengthTextBox");
		jText.setEditable( editable );
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("RadiusLabel");
		jLabel.setText( resource.translate("SwitchProperties.radius") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRadius();
		jText.setName("RadiusTextBox");
		jText.setEditable( editable );
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Radius2Label");
		jLabel.setText( resource.translate("SwitchProperties.radiusS") );
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRadius();
		jText.setName("Radius2TextBox");
		jText.setEditable( editable );
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("WidthLabel");
		jLabel.setText( resource.translate("SwitchProperties.width") );
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createWidth();
		jText.setName("WidthTextBox");
		jText.setEditable( editable );
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("FrictionLabel");
		jLabel.setText( resource.translate("SwitchProperties.friction") );
		c.gridx = 2;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createFriction();
		jText.setName("FrictionTextBox");
		jText.setColumns(7);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("TapLabel");
		jLabel.setText( resource.translate("SwitchProperties.tap") );
		c.gridx = 4;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTap();
		jText.setName("TapTextBox");
		jText.setColumns(7);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 5;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("QualityLabel");
		jLabel.setText( resource.translate("SwitchProperties.quality") );
		c.gridx = 4;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createQuality();
		jText.setName("QualityTextBox");
		jText.setColumns(7);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 5;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("DamageLabel");
		jLabel.setText( resource.translate("SwitchProperties.damage") );
		c.gridx = 4;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createDamage();
		jText.setName("DamageTextBox");
		jText.setColumns(7);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 5;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jPanel.setFocusTraversalPolicy(new GeneralTabFocusTraversal());
		jGeneralPanel.add(jPanel);

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("EnviromentLabel");
		jLabel.setText( resource.translate("SwitchProperties.enviroment") );
		c.gridx = 0;
		c.gridy = 0;
		c.insets = insets;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		JComboBox jComboBox = new JComboBox();
		jComboBox.setName("EnviromentComboBox");
		jComboBox.addItem(Track.BANK);
		jComboBox.addItem(Track.BRIDGE);
		jComboBox.addItem(Switch.CANYON);
		jComboBox.addItem(Switch.FLAT);
		jComboBox.addItem(Switch.MOUNTAINS);
		jComboBox.addItem(Switch.TUNNEL);
		jComboBox.setEnabled( editable );
		controls.putComponent(jComboBox);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jComboBox, c);
		jPanel.add(jComboBox);

		jGeneralPanel.add(jPanel);

		jTabbedPane.addTab(resource.translate("SwitchProperties.general"), jGeneralPanel);

		JPanel jTexturesPanel = new JPanel();
		jTexturesPanel.setName("TexturesPanel");
		jTexturesPanel.setBorder( BorderFactory.createEmptyBorder(3, 3, 3, 3) );
		jTexturesPanel.setLayout(new GridLayout(2, 1, 0, 5));

		/* Tekstury - grny panel */

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("VisibilityLabel");
		jLabel.setText( resource.translate("SwitchProperties.visibility") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		JPanel jVisibilityPanel = new JPanel();
		jVisibilityPanel.setBorder( BorderFactory.createEtchedBorder() );
		jVisibilityPanel.setLayout( new GridLayout(1, 2) );

		JRadioButton jRadioButton;
		ButtonGroup bGroup = new ButtonGroup();

		jRadioButton = new JRadioButton();
		jRadioButton.setName("VisibleRadioButton");
		jRadioButton.setText( resource.translate("SwitchProperties.visible") );
		jRadioButton.setEnabled( editable );
		controls.putComponent(jRadioButton);
		bGroup.add(jRadioButton);
		jVisibilityPanel.add(jRadioButton);

		jRadioButton = new JRadioButton();
		jRadioButton.setName("InvisibleRadioButton");
		jRadioButton.setText( resource.translate("SwitchProperties.invisible") );
		jRadioButton.setEnabled( editable );
		controls.putComponent(jRadioButton);
		bGroup.add(jRadioButton);
		jVisibilityPanel.add(jRadioButton);

		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jVisibilityPanel, c);
		jPanel.add(jVisibilityPanel);
		
		jTexturesPanel.add(jPanel);

		/* Tekstury - dolny panel */

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("TextureLabel");
		jLabel.setText( resource.translate("SwitchProperties.texture") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jComboBox = createTexture();
		jComboBox.setName("TextureComboBox");
		jComboBox.setEnabled( editable );
		controls.putComponent(jComboBox);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jComboBox, c);
		jPanel.add(jComboBox);

		jLabel = new JLabel();
		jLabel.setName("TextureLengthLabel");
		jLabel.setText( resource.translate("SwitchProperties.textureLength") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureLength();
		jText.setName("TextureLengthEditBox");
		jText.setEditable( editable );
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("SecondTextureLabel");
		jLabel.setText( resource.translate("SwitchProperties.secondTexture") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jComboBox = createSecondTexture();
		jComboBox.setName("SecondTextureComboBox");
		jComboBox.setEnabled( editable );
		controls.putComponent( jComboBox );
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jComboBox, c);
		jPanel.add(jComboBox);

		jLabel = new JLabel();
		jLabel.setName("TextureWidthLabel");
		jLabel.setText( resource.translate("SwitchProperties.textureWidth") );
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureWidth();
		jText.setName("TextureWidthTextBox");
		jText.setEditable( editable );
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("TextureHeightLabel");
		jLabel.setText( resource.translate("SwitchProperties.textureHeight") );
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureHeight();
		jText.setName("TextureHeightTextBox");
		jText.setEditable( editable );
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("TextureSlopeLabel");
		jLabel.setText( resource.translate("SwitchProperties.textureSlope") );
		c.gridx = 2;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureSlope();
		jText.setName("TextureSlopeTextBox");
		jText.setEditable( editable );
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 3;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jTexturesPanel.add(jPanel);

		jTabbedPane.addTab(resource.translate("SwitchProperties.textures"), jTexturesPanel);

		/* Geometria */
		JPanel jTempPanel = new JPanel();
		jTempPanel.setLayout(new GridBagLayout());
		GridBagConstraints c3 = new GridBagConstraints();
		c3.gridx = 0; c3.gridy = 0; c3.fill = GridBagConstraints.BOTH;

		JPanel jGeometryPanel = new JPanel();
		jGeometryPanel.setName("GeometryPanel");
		jGeometryPanel.setBorder( BorderFactory.createEmptyBorder(3, 3, 3, 3) );
		GridBagLayout gb2 = new GridBagLayout();
		GridBagConstraints c2 = new GridBagConstraints();
		jGeometryPanel.setLayout(gb2);

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("Point1Label");
		jLabel.setText( resource.translate("SwitchProperties.point1") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Point1XTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point1YTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point1ZTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Vector1Label");
		jLabel.setText( resource.translate("SwitchProperties.vector1") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Vector1XTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector1YTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector1ZTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Roll1Label");
		jLabel.setText( resource.translate("SwitchProperties.roll1") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRoll();
		jText.setName("Roll1TextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		c2.gridx = 0;
		c2.gridy = 0;
		c2.insets = new Insets(0, 0, 5, 5);
		gb2.setConstraints(jPanel, c2);
		jGeometryPanel.add(jPanel);

		/* Point 2 */
		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("Vector2Label");
		jLabel.setText( resource.translate("SwitchProperties.vector2") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Vector2XTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector2YTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector2ZTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Point2Label");
		jLabel.setText( resource.translate("SwitchProperties.point2") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Point2XTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point2YTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point2ZTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Roll2Label");
		jLabel.setText( resource.translate("SwitchProperties.roll2") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRoll();
		jText.setName("Roll2TextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		c2.gridx = 0;
		c2.gridy = 1;
		c2.insets = new Insets(0, 0, 5, 5);
		gb2.setConstraints(jPanel, c2);
		jGeometryPanel.add(jPanel);

		/* Grade */
		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("GradeLabel");
		jLabel.setText( resource.translate("SwitchProperties.grade") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createGrade();
		jText.setName("GradeTextBox");
		jText.setColumns(6);
		jText.setEditable( false );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		c2.gridx = 0;
		c2.gridy = 2;
		c2.gridwidth = 2;
		c2.insets = new Insets(0, 0, 0, 0);
		int oldFill = c2.fill;
		c2.fill = GridBagConstraints.BOTH;
		gb2.setConstraints(jPanel, c2);
		jGeometryPanel.add(jPanel);
		c2.gridwidth = 1;
		c2.fill = oldFill;

		/* Point 3 */
		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("Point1RLabel");
		jLabel.setText( resource.translate("SwitchProperties.point1r") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Point1RXTextBox");
		jText.setColumns(6);
		jText.setEditable( false );
		jText.setDocument( ((JTextComponent) controls.getComponent("Point1XTextBox")).getDocument() );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point1RYTextBox");
		jText.setColumns(6);
		jText.setEditable( false );
		jText.setDocument( ((JTextComponent) controls.getComponent("Point1YTextBox")).getDocument() );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point1RZTextBox");
		jText.setColumns(6);
		jText.setEditable( false );
		jText.setDocument( ((JTextComponent) controls.getComponent("Point1ZTextBox")).getDocument() );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("VectorSLabel");
		jLabel.setText( resource.translate("SwitchProperties.vectorS") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("VectorSXTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("VectorSYTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("VectorSZTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("RollSLabel");
		jLabel.setText( resource.translate("SwitchProperties.rollS") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRoll();
		jText.setName("RollSTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		c2.gridx = 1;
		c2.gridy = 0;
		c2.insets = new Insets(0, 0, 5, 0);
		gb2.setConstraints(jPanel, c2);
		jGeometryPanel.add(jPanel);

		/* Point 4 */
		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("Vector3Label");
		jLabel.setText( resource.translate("SwitchProperties.vector3") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Vector3XTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector3YTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector3ZTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Point3Label");
		jLabel.setText( resource.translate("SwitchProperties.point3") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Point3XTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point3YTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point3ZTextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Roll3Label");
		jLabel.setText( resource.translate("SwitchProperties.roll3") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRoll();
		jText.setName("Roll3TextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		c2.gridx = 1;
		c2.gridy = 1;
		gb2.setConstraints(jPanel, c2);
		jGeometryPanel.add(jPanel);

		((GridBagLayout) jTempPanel.getLayout()).setConstraints(jGeometryPanel, c3);
		jTempPanel.add(jGeometryPanel);

		jTabbedPane.addTab(resource.translate("SwitchProperties.geometry"), jTempPanel);

		pane.add(jTabbedPane);

		return pane;
	}

	/**
	 * Fills in the form.
	 */
	protected void fillIn(Switch sswitch)
	{
		JTextField				jText;
		JFormattedTextField		jFormatText;
		JComboBox				jComboBox;
		JRadioButton			jRadioButton;

		jText = (JTextField) controls.getComponent("NameTextBox");
		jText.setText( sswitch.getName() );

		jFormatText = (JFormattedTextField) controls.getComponent("LengthTextBox");
		jFormatText.setValue( new Double(sswitch.getLength()) );

		jFormatText = (JFormattedTextField) controls.getComponent("RadiusTextBox");
		jFormatText.setValue( new Double(sswitch.getRadius()) );

		jFormatText = (JFormattedTextField) controls.getComponent("Radius2TextBox");
		jFormatText.setValue( new Double(sswitch.getSwitchedRadius()) );

		jFormatText = (JFormattedTextField) controls.getComponent("WidthTextBox");
		jFormatText.setValue( new Double(sswitch.getWidth()) );

		jFormatText = (JFormattedTextField) controls.getComponent("FrictionTextBox");
		jFormatText.setValue( new Double(sswitch.getFriction()) );

		jFormatText = (JFormattedTextField) controls.getComponent("TapTextBox");
		jFormatText.setValue( new Double(sswitch.getTap()) );

		jFormatText = (JFormattedTextField) controls.getComponent("QualityTextBox");
		jFormatText.setValue( new Integer(sswitch.getQuality()) );

		jFormatText = (JFormattedTextField) controls.getComponent("DamageTextBox");
		jFormatText.setValue( new Integer(sswitch.getDamage()) );

		jComboBox = (JComboBox) controls.getComponent("EnviromentComboBox");
		jComboBox.setSelectedItem( sswitch.getEnviroment() );

		if (sswitch.isVisible())
		{
			jRadioButton = (JRadioButton) controls.getComponent("VisibleRadioButton");
		}
		else
		{
			jRadioButton = (JRadioButton) controls.getComponent("InvisibleRadioButton");
		}
		jRadioButton.setSelected(true);

		jComboBox = (JComboBox) controls.getComponent("TextureComboBox");
		jComboBox.setSelectedItem( sswitch.getTexture() );

		jFormatText = (JFormattedTextField) controls.getComponent("TextureLengthEditBox");
		jFormatText.setValue( new Double(sswitch.getTextureLength()) );

		jComboBox = (JComboBox) controls.getComponent("SecondTextureComboBox");
		jComboBox.setSelectedItem( sswitch.getSecondTexture() );

		jFormatText = (JFormattedTextField) controls.getComponent("TextureWidthTextBox");
		jFormatText.setValue( new Double(sswitch.getTextureWidth()) );

		jFormatText = (JFormattedTextField) controls.getComponent("TextureHeightTextBox");
		jFormatText.setValue( new Double(sswitch.getTextureHeight()) );

		jFormatText = (JFormattedTextField) controls.getComponent("TextureSlopeTextBox");
		jFormatText.setValue( new Double(sswitch.getTextureSlope()) );

		jFormatText = (JFormattedTextField) controls.getComponent("Point1XTextBox");
		jFormatText.setValue( new Double(sswitch.getPoint1().x) );

		jFormatText = (JFormattedTextField) controls.getComponent("Point1YTextBox");
		jFormatText.setValue( new Double(sswitch.getPoint1().y) );

		jFormatText = (JFormattedTextField) controls.getComponent("Point1ZTextBox");
		jFormatText.setValue( new Double(sswitch.getPoint1().z) );

		jFormatText = (JFormattedTextField) controls.getComponent("Vector1XTextBox");
		jFormatText.setValue( new Double(sswitch.getVector1().x) );

		jFormatText = (JFormattedTextField) controls.getComponent("Vector1YTextBox");
		jFormatText.setValue( new Double(sswitch.getVector1().y) );

		jFormatText = (JFormattedTextField) controls.getComponent("Vector1ZTextBox");
		jFormatText.setValue( new Double(sswitch.getVector1().z) );

		jFormatText = (JFormattedTextField) controls.getComponent("Vector2XTextBox");
		jFormatText.setValue( new Double(sswitch.getVector2().x) );

		jFormatText = (JFormattedTextField) controls.getComponent("Vector2YTextBox");
		jFormatText.setValue( new Double(sswitch.getVector2().y) );

		jFormatText = (JFormattedTextField) controls.getComponent("Vector2ZTextBox");
		jFormatText.setValue( new Double(sswitch.getVector2().z) );

		jFormatText = (JFormattedTextField) controls.getComponent("Point2XTextBox");
		jFormatText.setValue( new Double(sswitch.getPoint2().x) );

		jFormatText = (JFormattedTextField) controls.getComponent("Point2YTextBox");
		jFormatText.setValue( new Double(sswitch.getPoint2().y) );

		jFormatText = (JFormattedTextField) controls.getComponent("Point2ZTextBox");
		jFormatText.setValue( new Double(sswitch.getPoint2().z) );

		jFormatText = (JFormattedTextField) controls.getComponent("Roll1TextBox");
		jFormatText.setValue( new Double(sswitch.getRoll1()) );

		jFormatText = (JFormattedTextField) controls.getComponent("Roll2TextBox");
		jFormatText.setValue( new Double(sswitch.getRoll2()) );

		jFormatText = (JFormattedTextField) controls.getComponent("GradeTextBox");
		jFormatText.setValue( new Double(sswitch.getIncline()) );	

		jFormatText = (JFormattedTextField) controls.getComponent("Point1RXTextBox");
		jFormatText.setValue( new Double(sswitch.getPoint1().x) );

		jFormatText = (JFormattedTextField) controls.getComponent("Point1RYTextBox");
		jFormatText.setValue( new Double(sswitch.getPoint1().y) );

		jFormatText = (JFormattedTextField) controls.getComponent("Point1RZTextBox");
		jFormatText.setValue( new Double(sswitch.getPoint1().z) );

		jFormatText = (JFormattedTextField) controls.getComponent("VectorSXTextBox");
		jFormatText.setValue( new Double(sswitch.getSwitchedVector().x) );

		jFormatText = (JFormattedTextField) controls.getComponent("VectorSYTextBox");
		jFormatText.setValue( new Double(sswitch.getSwitchedVector().y) );

		jFormatText = (JFormattedTextField) controls.getComponent("VectorSZTextBox");
		jFormatText.setValue( new Double(sswitch.getSwitchedVector().z) );

		jFormatText = (JFormattedTextField) controls.getComponent("RollSTextBox");
		jFormatText.setValue( new Double(sswitch.getSwitchedRoll()) );

		jFormatText = (JFormattedTextField) controls.getComponent("Vector3XTextBox");
		jFormatText.setValue( new Double(sswitch.getVector3().x) );

		jFormatText = (JFormattedTextField) controls.getComponent("Vector3YTextBox");
		jFormatText.setValue( new Double(sswitch.getVector3().y) );

		jFormatText = (JFormattedTextField) controls.getComponent("Vector3ZTextBox");
		jFormatText.setValue( new Double(sswitch.getVector3().z) );

		jFormatText = (JFormattedTextField) controls.getComponent("Point3XTextBox");
		jFormatText.setValue( new Double(sswitch.getPoint3().x) );

		jFormatText = (JFormattedTextField) controls.getComponent("Point3YTextBox");
		jFormatText.setValue( new Double(sswitch.getPoint3().y) );

		jFormatText = (JFormattedTextField) controls.getComponent("Point3ZTextBox");
		jFormatText.setValue( new Double(sswitch.getPoint3().z) );

		jFormatText = (JFormattedTextField) controls.getComponent("Roll3TextBox");
		jFormatText.setValue( new Double(sswitch.getRoll3()) );

		if (editable)
		{
			oldSwitch = sswitch;

			setConstraints();
		}
	}

	/**
	 * Gets controls map.
	 */
	public ControlMap getControls()
	{
		return controls;
	}

	/**
	 * Sets the constraints for text fields.
	 */
	private void setConstraints()
	{
		JFormattedTextField jFormatText;
		
		jFormatText = (JFormattedTextField) controls.getComponent("Roll1TextBox");
		setConstraints(jFormatText, new Double(-Track.ROLL), new Double(Track.ROLL));
		jFormatText = (JFormattedTextField) controls.getComponent("Roll2TextBox");
		setConstraints(jFormatText, new Double(-Track.ROLL), new Double(Track.ROLL));
		jFormatText = (JFormattedTextField) controls.getComponent("RollSTextBox");
		setConstraints(jFormatText, new Double(-Track.ROLL), new Double(Track.ROLL));
		jFormatText = (JFormattedTextField) controls.getComponent("Roll3TextBox");
		setConstraints(jFormatText, new Double(-Track.ROLL), new Double(Track.ROLL));
	}

	/*
	 * --------- Inner classes ---------
	 */
	
	class GeneralTabFocusTraversal extends FocusTraversalPolicy
	{
		public Component getComponentAfter(Container container, Component aComponent)
		{
			System.out.println(container);

			String name = aComponent.getName();
			if (name.equals("NameTextBox"))
				return controls.getComponent("LengthTextBox");
			else if (name.equals("RadiusTextBox"))
				return controls.getComponent("WidthTextBox");
			else if (name.equals("LengthTextBox"))
				return controls.getComponent("RadiusTextBox");
			else if (name.equals("WidthTextBox"))
				return controls.getComponent("FrictionTextBox");
			else if (name.equals("FrictionTextBox"))
				return controls.getComponent("TapTextBox");
			else if (name.equals("TapTextBox"))
				return controls.getComponent("QualityTextBox");
			else if (name.equals("QualityTextBox"))
				return controls.getComponent("DamageTextBox");

			return null;
		}

		public Component getComponentBefore(Container container, Component aComponent)
		{
			String name = aComponent.getName();
			if (name.equals("LengthTextBox"))
				return controls.getComponent("NameTextBox");
			else if (name.equals("RadiusTextBox"))
				return controls.getComponent("LengthTextBox");
			else if (name.equals("WidthTextBox"))
				return controls.getComponent("RadiusTextBox");
			else if (name.equals("FrictionTextBox"))
				return controls.getComponent("WidthTextBox");
			else if (name.equals("TapTextBox"))
				return controls.getComponent("FrictionTextBox");
			else if (name.equals("QualityTextBox"))
				return controls.getComponent("TapTextBox");
			else if (name.equals("DamageTextBox"))
				return controls.getComponent("QualityTextBox");

			return null;
		}

		public Component getDefaultComponent(Container container)
		{
			return controls.getComponent("NameTextBox");
		}

		public Component getFirstComponent(Container container)
		{
			return controls.getComponent("NameTextBox");
		}

		public Component getLastComponent(Container conatainer)
		{
			return controls.getComponent("DamageTextBox");
		}
	}

	/*
	 * ----------- OK action ------------
	 */
	
	class OKAction extends AbstractAction
	{
		public void actionPerformed(ActionEvent ae)
		{
			double					d;
			int						i;
			Point3d					p3d;
			Vector3d				v3d;
			JFormattedTextField		jFormatText;
			JTextField				jText;
			JComboBox				jComboBox;
			JRadioButton			jButton;

			Switch sswitch = new Switch();

			// Name
			jText = (JTextField) controls.getComponent("NameTextBox");
			String name = jText.getText();
			if (name.equals(""))
				name = null;
			sswitch.setName( name );

			// Length
			jFormatText = (JFormattedTextField) controls.getComponent("LengthTextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setLength( d );

			// Radius
			jFormatText = (JFormattedTextField) controls.getComponent("RadiusTextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setRadius( d );

			// Second Radius
			jFormatText = (JFormattedTextField) controls.getComponent("Radius2TextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setSwitchedRadius( d );

			// Width
			jFormatText = (JFormattedTextField) controls.getComponent("WidthTextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setWidth( d );

			// Tap
			jFormatText = (JFormattedTextField) controls.getComponent("TapTextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setTap( d );

			// Friction
			jFormatText = (JFormattedTextField) controls.getComponent("FrictionTextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setFriction( d );

			// Damage
			jFormatText = (JFormattedTextField) controls.getComponent("DamageTextBox");
			i = ((Number) jFormatText.getValue()).intValue();
			sswitch.setDamage( i );

			// Quality
			jFormatText = (JFormattedTextField) controls.getComponent("QualityTextBox");
			i = ((Number) jFormatText.getValue()).intValue();
			sswitch.setQuality( i );

			// Enviroment
			jComboBox = (JComboBox) controls.getComponent("EnviromentComboBox");
			sswitch.setEnviroment( (String) jComboBox.getSelectedItem() );

			// Visibility
			jButton = (JRadioButton) controls.getComponent("VisibleRadioButton");
			sswitch.setVisible( jButton.isSelected() );

			// Texture
			jComboBox = (JComboBox) controls.getComponent("TextureComboBox");
			sswitch.setTexture( (String) jComboBox.getSelectedItem() );

			// Second texture
			jComboBox = (JComboBox) controls.getComponent("SecondTextureComboBox");
			sswitch.setSecondTexture( (String) jComboBox.getSelectedItem() );

			// Texture length
			jFormatText = (JFormattedTextField) controls.getComponent("TextureLengthEditBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setTextureLength(d);

			// Texture width
			jFormatText = (JFormattedTextField) controls.getComponent("TextureWidthTextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setTextureWidth(d);

			// Texture height
			jFormatText = (JFormattedTextField) controls.getComponent("TextureHeightTextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setTextureHeight(d);

			// Texture slope
			jFormatText = (JFormattedTextField) controls.getComponent("TextureSlopeTextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setTextureSlope(d);

			// Roll 1
			jFormatText = (JFormattedTextField) controls.getComponent("Roll1TextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setRoll1(d);

			// Roll 2
			jFormatText = (JFormattedTextField) controls.getComponent("Roll2TextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setRoll2(d);

			// Roll S
			jFormatText = (JFormattedTextField) controls.getComponent("RollSTextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setSwitchedRoll(d);

			// Roll 3
			jFormatText = (JFormattedTextField) controls.getComponent("Roll3TextBox");
			d = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setRoll3(d);

			// Point 1
			p3d = new Point3d();
			jFormatText = (JFormattedTextField) controls.getComponent("Point1XTextBox");
			p3d.x = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Point1YTextBox");
			p3d.y = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Point1ZTextBox");
			p3d.z = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setPoint1( p3d );

			// Vector 1
			v3d = new Vector3d();
			jFormatText = (JFormattedTextField) controls.getComponent("Vector1XTextBox");
			v3d.x = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Vector1YTextBox");
			v3d.y = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Vector1ZTextBox");
			v3d.z = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setVector1( v3d );

			// Vector 2
			v3d = new Vector3d();
			jFormatText = (JFormattedTextField) controls.getComponent("Vector2XTextBox");
			v3d.x = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Vector2YTextBox");
			v3d.y = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Vector2ZTextBox");
			v3d.z = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setVector2( v3d );

			// Point 2
			p3d = new Point3d();
			jFormatText = (JFormattedTextField) controls.getComponent("Point2XTextBox");
			p3d.x = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Point2YTextBox");
			p3d.y = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Point2ZTextBox");
			p3d.z = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setPoint2( p3d );

			// Point 3
			p3d = new Point3d();
			jFormatText = (JFormattedTextField) controls.getComponent("Point3XTextBox");
			p3d.x = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Point3YTextBox");
			p3d.y = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Point3ZTextBox");
			p3d.z = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setPoint3( p3d );

			// Vector 3
			v3d = new Vector3d();
			jFormatText = (JFormattedTextField) controls.getComponent("Vector3XTextBox");
			v3d.x = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Vector3YTextBox");
			v3d.y = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("Vector3ZTextBox");
			v3d.z = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setVector3( v3d );

			// Switched vector
			v3d = new Vector3d();
			jFormatText = (JFormattedTextField) controls.getComponent("VectorSXTextBox");
			v3d.x = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("VectorSYTextBox");
			v3d.y = ((Number) jFormatText.getValue()).doubleValue();
			jFormatText = (JFormattedTextField) controls.getComponent("VectorSZTextBox");
			v3d.z = ((Number) jFormatText.getValue()).doubleValue();
			sswitch.setSwitchedVector( v3d );

			if (!sswitch.equals(oldSwitch))
			{
				// Odznacz stary tor.
				editor.clearSelection();

				Object[] oldArray = new Object[1];
				oldArray[0] = oldSwitch;
				Object[] newArray = new Object[1];
				newArray[0] = sswitch;

				editor.getScenery().replace(oldArray, newArray);

				// Zaznacz nowy tor.
				editor.setSelectedObject(sswitch);
			}

			dispose();
		}
	}

	class CancelAction extends AbstractAction
	{
		public void actionPerformed(ActionEvent ae)
		{
			dispose();
		}
	}

	/*
	 * ----------- Members ------------
	 */
	
	private Switch oldSwitch;

	private JSceneryEditor editor;
}
